﻿using DbContextLib;
using Model.DTO;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Model.Mappers;
using Model.Business;
using Microsoft.EntityFrameworkCore;
using Entities.SQLudeoDB;

namespace Services
{
    public class UserDataService : IDataService
    {
        private UserDbContext DbContext { get; set; }
        public UserDataService(UserDbContext context) 
        {
            DbContext = context;
            context.Database.EnsureCreated();
        }

        public UserDTO GetUserById(int id)
        {
            var userEntity = DbContext.Users.FirstOrDefault(u => u.Id == id);
            if (userEntity == null)
            {
                throw new ArgumentException("Impossible de trouver l'utilisateur", nameof(id));
            }
            return userEntity.FromEntityToModel().FromModelToDTO();

        }

        public UserDTO GetUserByUsername(string username)
        {
            var userEntity = DbContext.Users.FirstOrDefault(u => u.Username == username);
            if (userEntity == null)
            {
                throw new ArgumentException("Impossible de trouver l'utilisateur", nameof(username));
            }
            return userEntity.FromEntityToModel().FromModelToDTO();
        }

        public IEnumerable<UserDTO> GetUsers(int page, int number)
        {
            return DbContext.Users.Skip((page - 1) * number).Take(number).ToList().Select(u => u.FromEntityToModel().FromModelToDTO());
        }

        public bool DeleteUser(int id)
        {
            var userEntity = DbContext.Users.FirstOrDefault(u => u.Id == id);
            if (userEntity == null)
            {
                return false;
            }
            DbContext.Users.Remove(userEntity);
            DbContext.SaveChangesAsync();
            return true;
        }

        public UserDTO UpdateUser(int id, UserDTO user)
        {
            var updatingUser = DbContext.Users.FirstOrDefault(u => u.Id == id);
            if (updatingUser == null)
            {
                throw new ArgumentException("Impossible de trouver l'utilisateur", nameof(id));
            }
            updatingUser.Username = user.Username;
            updatingUser.Password = user.Password;
            updatingUser.Email = user.Email;
            updatingUser.IsAdmin = user.IsAdmin;
            // Permet d'indiquer en Db que l'entité a été modifiée.
            DbContext.Entry(updatingUser).State = EntityState.Modified;
            DbContext.SaveChangesAsync();
            return updatingUser.FromEntityToModel().FromModelToDTO();
        }

        public UserDTO CreateUser(string username, string password, string email, bool isAdmin)
        {
            var newUserEntity = new UserDTO
            {
                Username = username,
                Password = password,
                Email = email,
                IsAdmin = isAdmin
            };
            DbContext.Users.Add(newUserEntity.FromDTOToModel().FromModelToEntity());
            DbContext.SaveChangesAsync();
            return newUserEntity;
        }

        public IEnumerable<InquiryDTO> GetInquiries(int page, int number)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO GetInquiryById(int id)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO GetInquiryByTitle(string title)
        {
            throw new NotImplementedException();
        }
        public UserDTO CreateInquiry(string title, string description, bool isUser, InquiryTableEntity database, SolutionEntity inquiryTable)
        {
            throw new NotImplementedException();
        }

        public bool DeleteInquiry(int id)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO UpdateInquiry(int id, InquiryDTO inquiry)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<InquiryDTO> GetLessons(int page, int number)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO GetLessonById(int id)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO GetLessonByTitle(string title)
        {
            throw new NotImplementedException();
        }

        public bool DeleteLesson(int id)
        {
            throw new NotImplementedException();
        }

        public InquiryDTO UpdateLesson(int id, LessonDTO lesson)
        {
            throw new NotImplementedException();
        }

        public UserDTO CreateLesson(string title, string lastPublisher, DateOnly lastEdit)
        {
            throw new NotImplementedException();
        }
    }
}
